#include "konst.ui.text.h"

textbrowser::textbrowser(int nx1, int ny1, int nx2, int ny2, int clr) {
    setcoord(nx1, ny1, nx2, ny2);
    setcolor(clr);
    idle = 0;
    otherkeys = 0;
}

textbrowser::textbrowser(int clr = 0) {
    setcolor(clr);
    idle = 0;
    otherkeys = 0;
}

textbrowser::~textbrowser() {
}

void textbrowser::setcolor(int clr = 0) {
    ncolor = clr;
}

void textbrowser::setcoord(int nx1, int ny1, int nx2, int ny2) {
    x1 = nx1;
    x2 = nx2;
    y1 = ny1;
    y2 = ny2;
}

void textbrowser::redraw() {
    draw(line);
}

bool textbrowser::empty() {
    return lines.empty();
}

void textbrowser::draw(int line) {
    int i;
    attrset(ncolor);

    for(i = line; (i-line < y2-y1) && (i < lines.size()); i++) {
        mvprintw(y1+i-line, x1, "");
        printstring(lines[i]);
        for(int k = lines[i].size(); k < x2-x1; k++) printw(" ");
    }

    for(; i-line < y2-y1; i++) mvhline(y1+i-line, x1, ' ', x2-x1);
    refresh();
}

void textbrowser::setbuf(const string p) {
    line = 0;
    formattext(p, lines, true, x2-x1);
    if((endline = lines.size()-y2+y1+1) < 0) endline = 0;
}

int textbrowser::open() {
    bool finished = false, go;
    int res, ch;
    
    savescr();
    draw(line);

    while(!finished) {
        if(idle) go = keypressed(); else go = true;

        if(go) switch(ch = getkey()) {
            case KEY_UP:
            case KEY_DOWN:
            case KEY_PPAGE:
            case KEY_NPAGE:
            case KEY_END:
            case KEY_HOME:
                move(ch);
                draw(line);
                break;
            case KEY_F(10):
            case 27:
                res = 0;
                finished = true;
                break;
                
            default:
                if(otherkeys) {
                    res = (*otherkeys)(this, ch);
                    if(res >= 0) finished = true;
                }

        } else {
            if(idle) (*idle)(this);
        }
    }
    
    return res;
}

void textbrowser::move(int k) {
    switch(k) {
        case KEY_UP: if(--line < 0) line = 0; break;
        case KEY_DOWN: if(++line > endline) line = endline; break;
        case KEY_PPAGE: if((line -= y2-y1) < 0) line = 0; break;
        case KEY_NPAGE: if((line += y2-y1) > endline) line = endline; break;
        case KEY_END: line = endline; break;
        case KEY_HOME: line = 0; break;
    }
}

